"use strict";

var _ = require(".");

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

describe("**Feature**: when provided one argument (type), it returns an action creator function for that type", function () {
  describe("GIVEN an action type constant, 'ACTION_TYPE_CONSTANT'", function () {
    var ACTION_TYPE_CONSTANT = "ACTION_TYPE_CONSTANT";
    describe("WHEN it is called with that constant", function () {
      var actionCreator = (0, _.makeActionCreator)(ACTION_TYPE_CONSTANT);
      test("THEN it returns a function with type 'ACTION_TYPE_CONSTANT'", function () {
        expect(_typeof(actionCreator)).toBe("function");
        expect(actionCreator.type).toBe(ACTION_TYPE_CONSTANT);
      });
      describe("AND the returned function is executed with no arguments", function () {
        var result = actionCreator();
        test("THEN the result is { type: 'ACTION_TYPE_CONSTANT' }", function () {
          // Check overall object
          expect(result).toEqual({
            type: ACTION_TYPE_CONSTANT
          }); // Check type for referential identity

          expect(result.type).toBe(ACTION_TYPE_CONSTANT);
        });
      });
      describe("AND the returned function is executed with one argument: 'arg one'", function () {
        var argOne = "arg one";
        var result = actionCreator(argOne);
        test("THEN the result is { type: 'ACTION_TYPE_CONSTANT', payload: 'arg one' }", function () {
          // Check overall object
          expect(result).toEqual({
            type: ACTION_TYPE_CONSTANT,
            payload: argOne
          }); // Check type and payload for referential identity

          expect(result.type).toBe(ACTION_TYPE_CONSTANT);
          expect(result.payload).toBe(argOne);
        });
      });
      describe("AND the returned function is executed with two arguments: 'arg one', 'arg two'", function () {
        var argOne = "arg one";
        var argTwo = "arg two";
        var result = actionCreator(argOne, argTwo);
        test("THEN the result is { type: 'ACTION_TYPE_CONSTANT', payload: 'arg one', meta: 'arg two' }", function () {
          // Check overall object
          expect(result).toEqual({
            type: ACTION_TYPE_CONSTANT,
            payload: argOne,
            meta: argTwo
          }); // Check type, payload and meta for referential identity

          expect(result.type).toBe(ACTION_TYPE_CONSTANT);
          expect(result.payload).toBe(argOne);
          expect(result.meta).toBe(argTwo);
        });
      });
      describe("AND the returned function is executed with three arguments: 'arg one', 'arg two', { arg: 'three' }", function () {
        var argOne = "arg one";
        var argTwo = "arg two";
        var three = 'three';
        var argThree = {
          arg: three
        };
        var result = actionCreator(argOne, argTwo, argThree);
        test("THEN the result is { type: 'ACTION_TYPE_CONSTANT', payload: 'arg one', meta: 'arg two', arg: 'three' }", function () {
          // Check overall object
          expect(result).toEqual({
            type: ACTION_TYPE_CONSTANT,
            payload: argOne,
            meta: argTwo,
            arg: three
          }); // Check type, payload, meta for referential identity

          expect(result.type).toBe(ACTION_TYPE_CONSTANT);
          expect(result.payload).toBe(argOne);
          expect(result.meta).toBe(argTwo); // Check arg for referential identity

          expect(result.arg).toBe(three);
        });
      });
    });
  });
});