"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.actionsFor = void 0;

var _lodash = _interopRequireDefault(require("lodash"));

var _array = require("./array");

var _object = require("./object");

var _boolean = require("./boolean");

var _number = require("./number");

var _any = require("./any");

var _forString = require("./string/forString");

var _custom = require("../custom");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance"); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var actionsFor = function actionsFor(stateShape, customReducers) {
  var paths = recursivelyGeneratePaths(stateShape);
  var actions = {
    create: createFor(stateShape, customReducers)
  };
  paths.forEach(function (path) {
    var isPathToBranch = isBranch(_lodash["default"].get(stateShape, path));

    if (isPathToBranch) {
      addActionsToBranch(actions, path, stateShape, customReducers);
    } else {
      addActionsToLeaf(actions, path, stateShape, customReducers);
    }
  });
  return actions;
};

exports.actionsFor = actionsFor;

var actionsForLeafOrBranch = function actionsForLeafOrBranch(leafOrBranch) {
  var pathToLeafOrBranch = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
  var stateShape = arguments.length > 2 ? arguments[2] : undefined;
  var customReducers = arguments.length > 3 ? arguments[3] : undefined;
  leafOrBranch.create = createFor(stateShape, customReducers, pathToLeafOrBranch);
  return leafOrBranch;
};

var addActionsToBranch = function addActionsToBranch(actions, path, stateShape, customReducers) {
  var branch = _lodash["default"].get(stateShape, path);

  _lodash["default"].set(actions, path, actionsForLeafOrBranch(branch, path, stateShape, customReducers));
};

var addActionsToLeaf = function addActionsToLeaf(actions, path, stateShape, customReducers) {
  _lodash["default"].set(actions, path, actionsForLeafOrBranch({}, path, stateShape, customReducers));
};

var createFor = function createFor(stateShape, customReducers) {
  var pathToLeafOrBranch = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];
  var initialState = pathToLeafOrBranch.length >= 1 ? _lodash["default"].get(stateShape, pathToLeafOrBranch) : stateShape;
  var actionCreators;
  var basicActionCreators = (0, _any.forAny)(pathToLeafOrBranch);
  var asArray = (0, _array.forArray)(pathToLeafOrBranch);
  var asBoolean = (0, _boolean.forBoolean)(pathToLeafOrBranch);
  var asNumber = (0, _number.forNumber)(pathToLeafOrBranch);
  var asObject = (0, _object.forObject)(pathToLeafOrBranch);
  var asString = (0, _forString.forString)(pathToLeafOrBranch);
  var custom = (0, _custom.makeCustomActions)(customReducers, pathToLeafOrBranch);

  if (_lodash["default"].isBoolean(initialState)) {
    actionCreators = asBoolean;
  } else if (_lodash["default"].isArray(initialState)) {
    actionCreators = asArray;
  } else if (_lodash["default"].isNumber(initialState)) {
    actionCreators = asNumber;
  } else if (_lodash["default"].isPlainObject(initialState)) {
    actionCreators = asObject;
  } else if (_lodash["default"].isString(initialState)) {
    actionCreators = asString;
  }

  return _objectSpread({}, basicActionCreators, actionCreators, {
    asArray: asArray,
    asBoolean: asBoolean,
    asNumber: asNumber,
    asObject: asObject,
    asString: asString,
    custom: custom
  }, custom);
};

var isBranch = function isBranch(leafOrBranch) {
  return !Array.isArray(leafOrBranch) && _typeof(leafOrBranch) === "object" && _lodash["default"].size(leafOrBranch) >= 1;
};

var recursivelyGeneratePaths = function recursivelyGeneratePaths(stateShape) {
  var paths = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
  var currentPath = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];

  if (_lodash["default"].isPlainObject(stateShape)) {
    _lodash["default"].forEach(stateShape, function (val, key) {
      var newPath = [].concat(_toConsumableArray(currentPath), [key]);
      paths.push(newPath);
      recursivelyGeneratePaths(val, paths, newPath);
    });
  }

  return paths;
};