"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.leafReducer = void 0;

var _lodash = _interopRequireDefault(require("lodash"));

var _atomic = require("../actions/atomic");

var _condtions = require("../actions/condtions");

var _leafReducerArray = require("./array/leafReducerArray");

var _leafReducerObject = require("./object/leafReducerObject");

var _leafReducerString = require("./string/leafReducerString");

var _leafReducerBoolean = require("./boolean/leafReducerBoolean");

var _leafReducerNumber = require("./number/leafReducerNumber");

var _leafReducerCustom = require("./custom/leafReducerCustom");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var leafReducer = function leafReducer(leafState, action, wholeState, initialWhole, reducersDict) {
  var _action$leaf = action.leaf,
      leaf = _action$leaf === void 0 ? {} : _action$leaf,
      payload = action.payload;
  var condition = leaf.condition,
      custom = leaf.custom,
      creatorKey = leaf.creatorKey,
      path = leaf.path; // Custom actions

  if (custom) {
    return (0, _leafReducerCustom.leafReducerCustom)(reducersDict, leafState, action, wholeState);
  } // Type-specific actions


  switch (condition) {
    case _condtions.conditions.ARRAY:
      return (0, _leafReducerArray.leafReducerArray)(leafState, {
        creatorKey: creatorKey,
        payload: payload
      });

    case _condtions.conditions.BOOLEAN:
      return (0, _leafReducerBoolean.leafReducerBoolean)(leafState, {
        creatorKey: creatorKey
      });

    case _condtions.conditions.NUMBER:
      return (0, _leafReducerNumber.leafReducerNumber)(leafState, {
        creatorKey: creatorKey,
        payload: payload
      });

    case _condtions.conditions.OBJECT:
      return (0, _leafReducerObject.leafReducerObject)(leafState, {
        creatorKey: creatorKey,
        payload: payload
      });

    case _condtions.conditions.STRING:
      return (0, _leafReducerString.leafReducerString)(leafState, {
        creatorKey: creatorKey,
        payload: payload
      });
  } // Type-agnostic actions


  switch (creatorKey) {
    case _atomic.atomicActions.APPLY:
      return apply(payload, leafState, wholeState);

    case _atomic.atomicActions.CLEAR:
      return clear(leafState, payload);

    case _atomic.atomicActions.RESET:
      return reset(initialWhole, path);

    case _atomic.atomicActions.UPDATE:
      return payload;

    default:
      return leafState;
  }
};

exports.leafReducer = leafReducer;

var apply = function apply(callback, leafState, wholeState) {
  return callback(leafState, wholeState);
};

var clear = function clear(leafState, toNull) {
  if (toNull) {
    return null;
  } else if (_lodash["default"].isBoolean(leafState)) {
    return false;
  } else if (_lodash["default"].isString(leafState)) {
    return '';
  } else if (_lodash["default"].isArray(leafState)) {
    return [];
  } else if (_lodash["default"].isNumber(leafState)) {
    return 0;
  } else if (_lodash["default"].isPlainObject(leafState)) {
    return {};
  }
};

var reset = function reset(initialWholeState, path) {
  return _lodash["default"].size(path) >= 1 ? _lodash["default"].get(initialWholeState, path) : initialWholeState;
};

var update = function update(leafState, payload) {
  return payload;
};