"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.reduxLeaves = void 0;

var _lodash = _interopRequireDefault(require("lodash"));

var _immer = _interopRequireDefault(require("immer"));

var _utils = require("./utils");

var _actions = require("./actions/");

var _leafReducer = require("./leafReducer");

var _index = require("./utils/index");

var _standardiseReducersDict = require("./reducersDict/standardise/standardiseReducersDict");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var reduxLeaves = function reduxLeaves(initialState) {
  var reducersDict = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var leafReducersDict = (0, _standardiseReducersDict.standardiseReducersDict)(reducersDict);

  function immeredReducer() {
    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
    var action = arguments.length > 1 ? arguments[1] : undefined;
    return (0, _immer["default"])(state, function (draftState) {
      var _action$leaf = action.leaf,
          leaf = _action$leaf === void 0 ? {} : _action$leaf;
      var path = leaf.path,
          condition = leaf.condition,
          creatorKey = leaf.creatorKey,
          custom = leaf.custom;
      var prevLeafState = (0, _index.getState)(draftState, path);
      var newLeafState = (0, _leafReducer.leafReducer)(prevLeafState, action, draftState, initialState, leafReducersDict);
      return (0, _utils.updateState)(draftState, path, newLeafState);
    });
  }

  var actions = (0, _actions.actionsFor)(_lodash["default"].cloneDeep(initialState), leafReducersDict);
  return [immeredReducer, actions];
};

exports.reduxLeaves = reduxLeaves;